@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2015 jointly by the following organizations:
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -50,9 +50,7 @@
 #include <sbml/util/ElementFilter.h>
 
 /** @cond doxygenIgnored */
-
 using namespace std;
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_BEGIN
@@ -60,8 +58,6 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 
 Event::Event (unsigned int level, unsigned int version) :
    SBase ( level, version )
- , mId                       ( ""   )
- , mName                     ( ""   )
  , mTrigger                  ( NULL    )
  , mDelay                    ( NULL    )
  , mPriority                 ( NULL    )
@@ -69,8 +65,8 @@ Event::Event (unsigned int level, unsigned int version) :
  , mIsSetUseValuesFromTriggerTime ( false )
  , mExplicitlySetUVFTT (false )
  , mEventAssignments(level, version)
+ , mInternalId      ("")
 {
-  mInternalIdOnly = false;
   if (!hasValidLevelVersionNamespaceCombination())
     throw SBMLConstructorException();
   // before level 3 useValuesFromTriggerTime was set by default
@@ -84,8 +80,6 @@ Event::Event (unsigned int level, unsigned int version) :
 
 Event::Event (SBMLNamespaces * sbmlns) :
    SBase                     ( sbmlns )
- , mId                       ( ""   )
- , mName                     ( ""   )
  , mTrigger                  ( NULL    )
  , mDelay                    ( NULL    )
  , mPriority                 ( NULL    )
@@ -93,9 +87,8 @@ Event::Event (SBMLNamespaces * sbmlns) :
  , mIsSetUseValuesFromTriggerTime (false )
  , mExplicitlySetUVFTT (false )
  , mEventAssignments(sbmlns)
+ , mInternalId      ("")
 {
-  mInternalIdOnly = false;
-
   if (!hasValidLevelVersionNamespaceCombination())
   {
     throw SBMLConstructorException(getElementName(), sbmlns);
@@ -125,40 +118,34 @@ Event::~Event ()
  * Copy constructor. Creates a copy of this Event.
  */
 Event::Event (const Event& orig) :
-   SBase                     ( orig )
- , mTrigger                  ( NULL    )
- , mDelay                    ( NULL    )
- , mPriority                 ( NULL    )
- , mEventAssignments         ( orig.mEventAssignments         )
+   SBase                     ( orig            )
+ , mTrigger                  ( NULL            )
+ , mDelay                    ( NULL            )
+ , mPriority                 ( NULL            )
+ , mTimeUnits                ( orig.mTimeUnits )
+ , mUseValuesFromTriggerTime ( orig.mUseValuesFromTriggerTime )
+ , mIsSetUseValuesFromTriggerTime ( orig.mIsSetUseValuesFromTriggerTime )
+ , mExplicitlySetUVFTT       ( orig.mExplicitlySetUVFTT )
+ , mEventAssignments         ( orig.mEventAssignments   )
+ , mInternalId      ( orig.mInternalId      )
 {
-  if (&orig == NULL)
+  
+  if (orig.mTrigger != NULL) 
   {
-    throw SBMLConstructorException("Null argument to copy constructor");
+    mTrigger = new Trigger(*orig.getTrigger());
   }
-  else
+  
+  if (orig.mDelay != NULL) 
   {
-    mId                            = orig.mId;  
-    mName                          = orig.mName;
-    mTimeUnits                     = orig.mTimeUnits;
-    mUseValuesFromTriggerTime      = orig.mUseValuesFromTriggerTime ;
-    mIsSetUseValuesFromTriggerTime = orig.mIsSetUseValuesFromTriggerTime ;
-    mExplicitlySetUVFTT            = orig.mExplicitlySetUVFTT;
-    mInternalIdOnly                = orig.mInternalIdOnly;
- 
-    if (orig.mTrigger != NULL) 
-    {
-      mTrigger = new Trigger(*orig.getTrigger());
-    }
-    if (orig.mDelay != NULL) 
-    {
-      mDelay = new Delay(*orig.getDelay());
-    }
-    if (orig.mPriority != NULL) 
-    {
-      mPriority = new Priority(*orig.getPriority());
-    }
+    mDelay = new Delay(*orig.getDelay());
   }
   
+  if (orig.mPriority != NULL) 
+  {
+    mPriority = new Priority(*orig.getPriority());
+  }
+  
+  
   connectToChild();
 }
  
@@ -168,21 +155,15 @@ Event::Event (const Event& orig) :
  */
 Event& Event::operator=(const Event& rhs)
 {
-  if (&rhs == NULL)
-  {
-    throw SBMLConstructorException("Null argument to assignment operator");
-  }
-  else if(&rhs!=this)
+  if(&rhs!=this)
   {
     this->SBase::operator =(rhs);
    
-    mId = rhs.mId;
-    mName = rhs.mName;
     mTimeUnits        = rhs.mTimeUnits        ;
     mUseValuesFromTriggerTime = rhs.mUseValuesFromTriggerTime;
     mIsSetUseValuesFromTriggerTime = rhs.mIsSetUseValuesFromTriggerTime;
     mExplicitlySetUVFTT = rhs.mExplicitlySetUVFTT;
-    mInternalIdOnly   = rhs.mInternalIdOnly   ;
+    mInternalId     = rhs.mInternalId     ;
     mEventAssignments = rhs.mEventAssignments ;
 
     delete mTrigger;
@@ -222,13 +203,7 @@ Event& Event::operator=(const Event& rhs)
 }
 
 
-/*
- * Accepts the given SBMLVisitor.
- *
- * @return the result of calling <code>v.visit()</code>, which indicates
- * whether or not the Visitor would like to visit the Model's next Event
- * (if available).
- */
+/** @cond doxygenLibsbmlInternal */
 bool
 Event::accept (SBMLVisitor& v) const
 {
@@ -244,6 +219,7 @@ Event::accept (SBMLVisitor& v) const
 
   return result;
 }
+/** @endcond */
 
 
 /*
@@ -521,7 +497,7 @@ Event::isSetUseValuesFromTriggerTime () const
 
 
 /*
- * Sets the id of this SBML object to a copy of sid.
+ * Sets the id of this SBML object to a copy of @p sid.
  */
 int
 Event::setId (const std::string& sid)
@@ -535,18 +511,13 @@ Event::setId (const std::string& sid)
     return LIBSBML_UNEXPECTED_ATTRIBUTE;
   }
 */
-  if (&(sid) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (!(SyntaxChecker::isValidInternalSId(sid)))
+  if (!(SyntaxChecker::isValidInternalSId(sid)))
   {
     return LIBSBML_INVALID_ATTRIBUTE_VALUE;
   }
   else
   {
     mId = sid;
-    mInternalIdOnly = false;
     return LIBSBML_OPERATION_SUCCESS;
   }
 }
@@ -561,11 +532,7 @@ Event::setName (const std::string& name)
   /* if this is setting an L2 name the type is string
    * whereas if it is setting an L1 name its type is SId
    */
-  if (&(name) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (getLevel() == 1)
+  if (getLevel() == 1)
   {
     if (!(SyntaxChecker::isValidInternalSId(name)))
     {
@@ -698,7 +665,7 @@ Event::setPriority (const Priority* priority)
 
 
 /*
- * Sets the timeUnits of this Event to a copy of sid.
+ * Sets the timeUnits of this Event to a copy of @p sid.
  *
  * @warning Definitions of Event in SBML Level 2 Versions 1 and 2
  * included the additional attribute called "timeUnits", but it was
@@ -709,11 +676,7 @@ Event::setPriority (const Priority* priority)
 int
 Event::setTimeUnits (const std::string& sid)
 {
-  if (&(sid) == NULL)
-  {
-    return LIBSBML_INVALID_ATTRIBUTE_VALUE;
-  }
-  else if (getLevel() == 2 && getVersion() > 2)
+  if (getLevel() == 2 && getVersion() > 2)
   {
     return LIBSBML_UNEXPECTED_ATTRIBUTE;
   }
@@ -1103,25 +1066,23 @@ Event::getEventAssignment (unsigned int n)
 
 /*
  * @return the EventAssignment for the given variable, or @c NULL if no such
- * EventAssignment exits.
+ * EventAssignment exists.
  */
 const EventAssignment*
 Event::getEventAssignment (const std::string& variable) const
 {
-  return (&variable != NULL ) ? 
-  static_cast<const EventAssignment*>( mEventAssignments.get(variable) ) : NULL;
+  return static_cast<const EventAssignment*>( mEventAssignments.get(variable) );
 }
 
 
 /*
  * @return the EventAssignment for the given variable, or @c NULL if no such
- * EventAssignment exits.
+ * EventAssignment exists.
  */
 EventAssignment*
 Event::getEventAssignment (const std::string& variable)
 {
-  return (&variable != NULL ) ? 
-    static_cast<EventAssignment*>( mEventAssignments.get(variable) ) : NULL;
+  return static_cast<EventAssignment*>( mEventAssignments.get(variable) );
 }
 
 
@@ -1153,12 +1114,11 @@ Event::removeEventAssignment (unsigned int n)
 EventAssignment* 
 Event::removeEventAssignment (const std::string& variable)
 {
-  return (&variable != NULL) ? mEventAssignments.remove(variable) : NULL;  
+  return mEventAssignments.remove(variable);
 }
 
 
 /** @cond doxygenLibsbmlInternal */
-
 /*
  * Sets the parent SBMLDocument of this SBML object.
  */
@@ -1204,7 +1164,18 @@ Event::enablePackageInternal(const std::string& pkgURI, const std::string& pkgPr
   if (mPriority)   mPriority->enablePackageInternal(pkgURI,pkgPrefix,flag);
 }
 
+void
+Event::updateSBMLNamespace(const std::string& pkg, unsigned int level,
+  unsigned int version)
+{
+  SBase::updateSBMLNamespace(pkg, level, version);
 
+  mEventAssignments.updateSBMLNamespace(pkg, level, version);
+  if (mTrigger) mTrigger->updateSBMLNamespace(pkg, level, version);
+  if (mDelay)   mDelay->updateSBMLNamespace(pkg, level, version);
+  if (mPriority)   mPriority->updateSBMLNamespace(pkg, level, version);
+
+}
 /** @endcond */
 
 
@@ -1268,16 +1239,6 @@ Event::hasRequiredElements() const
 
 
 /** @cond doxygenLibsbmlInternal */
-
-/*
- * sets the mInternalIdOnly flag
- */
-void 
-Event::setInternalIdOnly()
-{
-  mInternalIdOnly = true;
-}
-
 /*
  * @return the SBML object corresponding to next XMLToken in the
  * XMLInputStream or @c NULL if the token was not recognized.
@@ -1294,11 +1255,12 @@ Event::createObject (XMLInputStream& stream)
     {
       if (getLevel() < 3)
         logError(NotSchemaConformant, getLevel(), getVersion(),
-	       "Only one <listOfEventAssignments> elements is permitted "
-	       "in a single <event> element.");
+         "Only one <listOfEventAssignments> elements is permitted "
+         "in a single <event> element.");
       else
         logError(OneListOfEventAssignmentsPerEvent, getLevel(), getVersion());
     }
+    mEventAssignments.setExplicitlyListed();
     object = &mEventAssignments;
   }
   else if (name == "trigger")
@@ -1307,8 +1269,8 @@ Event::createObject (XMLInputStream& stream)
     {
       if (getLevel() < 3)
         logError(NotSchemaConformant, getLevel(), getVersion(),
-	       "Only one <trigger> elements is permitted "
-	       "in a single <event> element.");
+         "Only one <trigger> elements is permitted "
+         "in a single <event> element.");
       else
         logError(MissingTriggerInEvent, getLevel(), getVersion());
     }
@@ -1337,8 +1299,8 @@ Event::createObject (XMLInputStream& stream)
     {
       if (getLevel() < 3)
         logError(NotSchemaConformant, getLevel(), getVersion(),
-	       "Only one <delay> element is permitted in a single "
-	       "<event> element.");
+         "Only one <delay> element is permitted in a single "
+         "<event> element.");
       else
         logError(OnlyOneDelayPerEvent, getLevel(), getVersion());
     }
@@ -1366,7 +1328,7 @@ Event::createObject (XMLInputStream& stream)
     {
       if (getLevel() < 3)
         logError(NotSchemaConformant, getLevel(), getVersion(),
-	       "Priority is not a valid component for this level/version.");
+         "Priority is not a valid component for this level/version.");
       else 
       {
         logError(OnlyOnePriorityPerEvent, getLevel(), getVersion());
@@ -1397,8 +1359,499 @@ Event::createObject (XMLInputStream& stream)
 /** @endcond */
 
 
+
+
+
+
 /** @cond doxygenLibsbmlInternal */
 
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::getAttribute(const std::string& attributeName, bool& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  if (attributeName == "useValuesFromTriggerTime")
+  {
+    value = getUseValuesFromTriggerTime();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::getAttribute(const std::string& attributeName, int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::getAttribute(const std::string& attributeName, double& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::getAttribute(const std::string& attributeName,
+                    unsigned int& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::getAttribute(const std::string& attributeName,
+                    std::string& value) const
+{
+  int return_value = SBase::getAttribute(attributeName, value);
+
+  if (return_value == LIBSBML_OPERATION_SUCCESS)
+  {
+    return return_value;
+  }
+
+  if (attributeName == "timeUnits")
+  {
+    value = getTimeUnits();
+    return_value = LIBSBML_OPERATION_SUCCESS;
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Gets the value of the "attributeName" attribute of this Event.
+ */
+//int
+//Event::getAttribute(const std::string& attributeName, const char* value) const
+//{
+//  int return_value = SBase::getAttribute(attributeName, value);
+//
+//  if (return_value == LIBSBML_OPERATION_SUCCESS)
+//  {
+//    return return_value;
+//  }
+//
+//  if (attributeName == "timeUnits")
+//  {
+//    value = getTimeUnits().c_str();
+//    return_value = LIBSBML_OPERATION_SUCCESS;
+//  }
+//
+//  return return_value;
+//}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Predicate returning @c true if this Event's attribute "attributeName" is
+ * set.
+ */
+bool
+Event::isSetAttribute(const std::string& attributeName) const
+{
+  bool value = SBase::isSetAttribute(attributeName);
+
+  if (attributeName == "useValuesFromTriggerTime")
+  {
+    value = isSetUseValuesFromTriggerTime();
+  }
+  else if (attributeName == "timeUnits")
+  {
+    value = isSetTimeUnits();
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::setAttribute(const std::string& attributeName, bool value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  if (attributeName == "useValuesFromTriggerTime")
+  {
+    return_value = setUseValuesFromTriggerTime(value);
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::setAttribute(const std::string& attributeName, int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::setAttribute(const std::string& attributeName, double value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::setAttribute(const std::string& attributeName, unsigned int value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::setAttribute(const std::string& attributeName,
+                    const std::string& value)
+{
+  int return_value = SBase::setAttribute(attributeName, value);
+
+  if (attributeName == "timeUnits")
+  {
+    return_value = setTimeUnits(value);
+  }
+
+  return return_value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Sets the value of the "attributeName" attribute of this Event.
+ */
+//int
+//Event::setAttribute(const std::string& attributeName, const char* value)
+//{
+//  int return_value = SBase::setAttribute(attributeName, value);
+//
+//  if (attributeName == "timeUnits")
+//  {
+//    return_value = setTimeUnits(value);
+//  }
+//
+//  return return_value;
+//}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Unsets the value of the "attributeName" attribute of this Event.
+ */
+int
+Event::unsetAttribute(const std::string& attributeName)
+{
+  int value = SBase::unsetAttribute(attributeName);
+
+  if (attributeName == "useValuesFromTriggerTime")
+  {
+    value = unsetUseValuesFromTriggerTime();
+  }
+  else if (attributeName == "timeUnits")
+  {
+    value = unsetTimeUnits();
+  }
+
+  return value;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Creates and returns an new "elementName" object in this Event.
+ */
+SBase*
+Event::createChildObject(const std::string& elementName)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "trigger")
+  {
+    return createTrigger();
+  }
+  else if (elementName == "priority")
+  {
+    return createPriority();
+  }
+  else if (elementName == "delay")
+  {
+    return createDelay();
+  }
+  else if (elementName == "eventAssignment")
+  {
+    return createEventAssignment();
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Adds an new "elementName" object in this Event.
+ */
+int
+Event::addChildObject(const std::string& elementName, const SBase* element)
+{
+  if (elementName == "trigger" && element->getTypeCode() == SBML_TRIGGER)
+  {
+    return setTrigger((const Trigger*)(element));
+  }
+  else if (elementName == "priority" && element->getTypeCode() == SBML_PRIORITY)
+  {
+    return setPriority((const Priority*)(element));
+  }
+  else if (elementName == "delay" && element->getTypeCode() == SBML_DELAY)
+  {
+    return setDelay((const Delay*)(element));
+  }
+  else if (elementName == "eventAssignment" && element->getTypeCode() == SBML_EVENT_ASSIGNMENT)
+  {
+    return addEventAssignment((const EventAssignment*)(element));
+  }
+
+  return LIBSBML_OPERATION_FAILED;
+}
+
+/** @endcond */
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Adds an new "elementName" object in this Event.
+ */
+SBase*
+Event::removeChildObject(const std::string& elementName, const std::string& id)
+{
+
+  if (elementName == "trigger")
+  {
+    Trigger* t = getTrigger();
+    if(unsetTrigger() == LIBSBML_OPERATION_SUCCESS)
+      return t;
+  }
+  else if (elementName == "priority")
+  {
+    Priority* t = getPriority();
+    if (unsetPriority() == LIBSBML_OPERATION_SUCCESS)
+      return t;
+  }
+  else if (elementName == "delay")
+  {
+    Delay* t = getDelay();
+    if (unsetDelay() == LIBSBML_OPERATION_SUCCESS)
+      return t;
+  }
+  else if (elementName == "eventAssignment")
+  {
+    return removeEventAssignment(id);
+  }
+
+  return NULL;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the number of "elementName" in this Event.
+ */
+unsigned int
+Event::getNumObjects(const std::string& elementName)
+{
+  unsigned int n = 0;
+
+  if (elementName == "trigger")
+  {
+    if (isSetTrigger())
+    {
+      return 1;
+    }
+  }
+  else if (elementName == "priority")
+  {
+    if (isSetPriority())
+    {
+      return 1;
+    }
+  }
+  else if (elementName == "delay")
+  {
+    if (isSetDelay())
+    {
+      return 1;
+    }
+  }
+  else if (elementName == "eventAssignment")
+  {
+    return getNumEventAssignments();
+  }
+
+  return n;
+}
+
+/** @endcond */
+
+
+
+/** @cond doxygenLibsbmlInternal */
+
+/*
+ * Returns the nth object of "objectName" in this Event.
+ */
+SBase*
+Event::getObject(const std::string& elementName, unsigned int index)
+{
+  SBase* obj = NULL;
+
+  if (elementName == "trigger")
+  {
+    return getTrigger();
+  }
+  else if (elementName == "priority")
+  {
+    return getPriority();
+  }
+  else if (elementName == "delay")
+  {
+    return getDelay();
+  }
+  else if (elementName == "eventAssignment")
+  {
+    return getEventAssignment(index);
+  }
+
+  return obj;
+}
+
+/** @endcond */
+
+
+
+
+/** @cond doxygenLibsbmlInternal */
 /**
  * Subclasses should override this method to get the list of
  * expected attributes.
@@ -1445,7 +1898,7 @@ Event::addExpectedAttributes(ExpectedAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 Event::readAttributes (const XMLAttributes& attributes,
@@ -1460,7 +1913,7 @@ Event::readAttributes (const XMLAttributes& attributes,
   {
   case 1:
     logError(NotSchemaConformant, level, version,
-	      "Event is not a valid component for this level/version.");
+        "Event is not a valid component for this level/version.");
     break;
   case 2:
     readL2Attributes(attributes);
@@ -1474,11 +1927,10 @@ Event::readAttributes (const XMLAttributes& attributes,
 /** @endcond */
 
 /** @cond doxygenLibsbmlInternal */
-
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 Event::readL2Attributes (const XMLAttributes& attributes)
@@ -1523,7 +1975,7 @@ Event::readL2Attributes (const XMLAttributes& attributes)
   //
   if (version == 2) 
     mSBOTerm = SBO::readTerm(attributes, this->getErrorLog(), level, version,
-				getLine(), getColumn());
+        getLine(), getColumn());
 
   //
   // useValuesFromTriggerTime: bool {use="optional" default="true"} (L2V4 ->)
@@ -1535,7 +1987,6 @@ Event::readL2Attributes (const XMLAttributes& attributes)
                                                 mUseValuesFromTriggerTime, getErrorLog(), false, getLine(), getColumn());
   }
 }
-
 /** @endcond */
 
 
@@ -1543,7 +1994,7 @@ Event::readL2Attributes (const XMLAttributes& attributes)
 /*
  * Subclasses should override this method to read values from the given
  * XMLAttributes set into their specific fields.  Be sure to call your
- * parents implementation of this method as well.
+ * parent's implementation of this method as well.
  */
 void
 Event::readL3Attributes (const XMLAttributes& attributes)
@@ -1554,19 +2005,29 @@ Event::readL3Attributes (const XMLAttributes& attributes)
   //
   // id: SId  { use="optional" }  (L2v1 ->)
   //
-  bool assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
-  if (assigned && mId.size() == 0)
+  // for l3v2 sbase will read this as generically optional
+  // we want to log errors relating to the specific object
+  if (version == 1)
   {
-    logEmptyString("id", level, version, "<event>");
+    bool assigned = attributes.readInto("id", mId, getErrorLog(), false, getLine(), getColumn());
+    if (assigned && mId.size() == 0)
+    {
+      logEmptyString("id", level, version, "<event>");
+    }
+    if (!SyntaxChecker::isValidInternalSId(mId)) 
+      logError(InvalidIdSyntax, level, version, "The id '" + mId + "' does not conform to the syntax.");
   }
-  if (!SyntaxChecker::isValidInternalSId(mId)) 
-    logError(InvalidIdSyntax, level, version, "The id '" + mId + "' does not conform to the syntax.");
 
   //
   // name: string  { use="optional" }  (L2v1 ->)
   //
-  attributes.readInto("name", mName, getErrorLog(), false, getLine(), getColumn());
-
+  // for l3v2 sbase will read this
+  if (version == 1)
+  {
+    attributes.readInto("name", mName, getErrorLog(), false, 
+                                       getLine(), getColumn());
+  }
+   
   //
   //
   // useValuesFromTriggerTime: bool {use="required" } (L3 ->)
@@ -1582,7 +2043,6 @@ Event::readL3Attributes (const XMLAttributes& attributes)
   }
 
 }
-
 /** @endcond */
 
 
@@ -1591,7 +2051,7 @@ Event::readL3Attributes (const XMLAttributes& attributes)
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write their XML attributes
- * to the XMLOutputStream.  Be sure to call your parents implementation
+ * to the XMLOutputStream.  Be sure to call your parent's implementation
  * of this method as well.
  */
 void
@@ -1619,17 +2079,20 @@ Event::writeAttributes (XMLOutputStream& stream) const
   }
 
 
-  //
-  //
-  // id: SId  { use="optional" }  (L2v1 ->)
-  //
-  if (!mInternalIdOnly)
+  // for L3V2 and above SBase will write this out
+  if (level < 3 || (level == 3 && version == 1))
+  {
+    //
+    //
+    // id: SId  { use="optional" }  (L2v1 ->)
+    //
     stream.writeAttribute("id", mId);
 
-  //
-  // name: string  { use="optional" }  (L2v1->)
-  //
-  stream.writeAttribute("name", mName);
+    //
+    // name: string  { use="optional" }  (L2v1->)
+    //
+    stream.writeAttribute("name", mName);
+  }
 
   if (level == 2 && version < 3)
   {
@@ -1669,7 +2132,7 @@ Event::writeAttributes (XMLOutputStream& stream) const
 /** @cond doxygenLibsbmlInternal */
 /*
  * Subclasses should override this method to write out their contained
- * SBML objects as XML elements.  Be sure to call your parents
+ * SBML objects as XML elements.  Be sure to call your parent's
  * implementation of this method as well.
  */
 void
@@ -1692,7 +2155,20 @@ Event::writeElements (XMLOutputStream& stream) const
     mPriority->write(stream);
   }
 
-  if ( getNumEventAssignments() > 0 ) mEventAssignments.write(stream);
+  if (getLevel() == 3 && getVersion() > 1)
+  {
+    if (mEventAssignments.hasOptionalElements() == true ||
+        mEventAssignments.hasOptionalAttributes() == true ||
+        mEventAssignments.isExplicitlyListed())
+    {
+      mEventAssignments.write(stream);
+    }
+  }
+  else
+  {
+    // use original code
+    if ( getNumEventAssignments() > 0 ) mEventAssignments.write(stream);
+  }
 
   //
   // (EXTENSION)
@@ -1774,11 +2250,11 @@ ListOfEvents::get(unsigned int n) const
  */
 struct IdEqE : public unary_function<SBase*, bool>
 {
-  const string& id;
+  const string& mId;
 
-  IdEqE (const string& id) : id(id) { }
+  IdEqE (const string& id) : mId(id) { }
   bool operator() (SBase* sb) 
-       { return static_cast <Event *> (sb)->getId() == id; }
+       { return static_cast <Event *> (sb)->getId() == mId; }
 };
 
 
@@ -1786,8 +2262,8 @@ struct IdEqE : public unary_function<SBase*, bool>
 Event*
 ListOfEvents::get (const std::string& sid)
 {
-  return (&sid != NULL) ? const_cast<Event*>( 
-    static_cast<const ListOfEvents&>(*this).get(sid) ) : NULL;
+  return const_cast<Event*>( 
+    static_cast<const ListOfEvents&>(*this).get(sid) );
 }
 
 
@@ -1795,7 +2271,6 @@ ListOfEvents::get (const std::string& sid)
 const Event*
 ListOfEvents::get (const std::string& sid) const
 {
-  if (&sid == NULL) return NULL;
   vector<SBase*>::const_iterator result;
 
   result = find_if( mItems.begin(), mItems.end(), IdEqE(sid) );
@@ -1818,15 +2293,12 @@ ListOfEvents::remove (const std::string& sid)
   SBase* item = NULL;
   vector<SBase*>::iterator result;
 
-  if (&(sid) != NULL)
-  {
-    result = find_if( mItems.begin(), mItems.end(), IdEqE(sid) );
+  result = find_if( mItems.begin(), mItems.end(), IdEqE(sid) );
 
-    if (result != mItems.end())
-    {
-      item = *result;
-      mItems.erase(result);
-    }
+  if (result != mItems.end())
+  {
+    item = *result;
+    mItems.erase(result);
   }
 
   return static_cast <Event*> (item);
@@ -1886,8 +2358,6 @@ ListOfEvents::createObject (XMLInputStream& stream)
 
 #endif /* __cplusplus */
 /** @cond doxygenIgnored */
-
-
 LIBSBML_EXTERN
 Event_t *
 Event_create (unsigned int level, unsigned int version)
@@ -2336,7 +2806,6 @@ ListOfEvents_removeById (ListOf_t *lo, const char *sid)
   else
     return NULL;
 }
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_END
