@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2014 jointly by the following organizations:
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -45,8 +45,8 @@ LIBSBML_CPP_NAMESPACE_BEGIN
 
 ExternalModelDefinition::ExternalModelDefinition (unsigned int level, unsigned int version, unsigned int pkgVersion) 
   : CompBase (level,version, pkgVersion)
-  , mId("")
-  , mName("")
+//  , mId("")
+//  , mName("")
   , mSource("")
   , mModelRef("")
   , mMd5("")
@@ -59,8 +59,8 @@ ExternalModelDefinition::ExternalModelDefinition (unsigned int level, unsigned i
 
 ExternalModelDefinition::ExternalModelDefinition(CompPkgNamespaces* compns)
   : CompBase(compns)
-  , mId("")
-  , mName("")
+//  , mId("")
+//  , mName("")
   , mSource("")
   , mModelRef("")
   , mMd5("")
@@ -323,7 +323,6 @@ ExternalModelDefinition::unsetMd5 ()
 int
 ExternalModelDefinition::setSource (const std::string& source)
 {
-  //LS DEBUG:  check syntax of source.  Somehow.  Maybe at least so that it's not ""?
   mSource = source;
   return LIBSBML_OPERATION_SUCCESS;
 }
@@ -412,20 +411,20 @@ ExternalModelDefinition::readAttributes (const XMLAttributes& attributes,
     static_cast<ListOfExternalModelDefinitions*>(getParentSBMLObject())->size() < 2)
   {
     unsigned int numErrs = getErrorLog()->getNumErrors();
-    for (int n = numErrs-1; n >= 0; n--)      
+    for (int n = (int)numErrs-1; n >= 0; --n)
     {
-      if (getErrorLog()->getError(n)->getErrorId() == UnknownPackageAttribute)
+      if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownPackageAttribute)
       {
         const std::string details = 
-          getErrorLog()->getError(n)->getMessage();
+          getErrorLog()->getError((unsigned int)n)->getMessage();
         getErrorLog()->remove(UnknownPackageAttribute);
         getErrorLog()->logPackageError("comp", CompLOExtModDefsAllowedAttributes,
           getPackageVersion(), sbmlLevel, sbmlVersion, details);
       } 
-      else if (getErrorLog()->getError(n)->getErrorId() == UnknownCoreAttribute)
+      else if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownCoreAttribute)
       {
         const std::string details = 
-          getErrorLog()->getError(n)->getMessage();
+          getErrorLog()->getError((unsigned int)n)->getMessage();
         getErrorLog()->remove(UnknownCoreAttribute);
         getErrorLog()->logPackageError("comp", CompLOExtModDefsAllowedAttributes,
           getPackageVersion(), sbmlLevel, sbmlVersion, details);
@@ -440,20 +439,20 @@ ExternalModelDefinition::readAttributes (const XMLAttributes& attributes,
   if (getErrorLog() != NULL)
   {
     unsigned int numErrs = getErrorLog()->getNumErrors();
-    for (int n = numErrs-1; n >= 0; n--)      
+    for (int n = (int)numErrs-1; n >= 0; n--)
     {
-      if (getErrorLog()->getError(n)->getErrorId() == UnknownPackageAttribute)
+      if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownPackageAttribute)
       {
         const std::string details = 
-          getErrorLog()->getError(n)->getMessage();
+          getErrorLog()->getError((unsigned int)n)->getMessage();
         getErrorLog()->remove(UnknownPackageAttribute);
         getErrorLog()->logPackageError("comp", CompExtModDefAllowedAttributes,
           getPackageVersion(), sbmlLevel, sbmlVersion, details);
       } 
-      else if (getErrorLog()->getError(n)->getErrorId() == UnknownCoreAttribute)
+      else if (getErrorLog()->getError((unsigned int)n)->getErrorId() == UnknownCoreAttribute)
       {
         const std::string details = 
-          getErrorLog()->getError(n)->getMessage();
+          getErrorLog()->getError((unsigned int)n)->getMessage();
         getErrorLog()->remove(UnknownCoreAttribute);
         getErrorLog()->logPackageError("comp", CompExtModDefAllowedCoreAttributes,
           getPackageVersion(), sbmlLevel, sbmlVersion, details);
@@ -508,7 +507,7 @@ ExternalModelDefinition::readAttributes (const XMLAttributes& attributes,
       if (!SyntaxChecker::isValidXMLanyURI(mSource)) 
       {
         getErrorLog()->logPackageError("comp", CompInvalidSourceSyntax,
-          getPackageVersion(), getLevel(), getVersion());
+          getPackageVersion(), getLevel(), getVersion(), "The source attribute value '" + mSource + "' does not conform to the anyURI syntax.");
       }
     }
     
@@ -577,13 +576,11 @@ ExternalModelDefinition::getTypeCode () const
 }
 
 /** @cond doxygenLibsbmlInternal */
-
 bool
 ExternalModelDefinition::accept (SBMLVisitor& v) const
 {
   return v.visit(*this);
 }
-
 /** @endcond */
 
 /*
@@ -926,31 +923,6 @@ ExternalModelDefinition_hasRequiredAttributes(ExternalModelDefinition_t * emd)
 {
   return (emd != NULL) ? static_cast<int>(emd->hasRequiredAttributes()) : 0;
 }
-
-
-LIBSBML_EXTERN
-ExternalModelDefinition_t *
-ListOfExternalModelDefinitions_getById(ListOf_t * lo, const char * sid)
-{
-  if (lo == NULL)
-    return NULL;
-
-  return (sid != NULL) ? static_cast <ListOfExternalModelDefinitions *>(lo)->get(sid) : NULL;
-}
-
-
-LIBSBML_EXTERN
-ExternalModelDefinition_t *
-ListOfExternalModelDefinitions_removeById(ListOf_t * lo, const char * sid)
-{
-  if (lo == NULL)
-    return NULL;
-
-  return (sid != NULL) ? static_cast <ListOfExternalModelDefinitions *>(lo)->remove(sid) : NULL;
-}
-
-
-
 /** @endcond */
 
 LIBSBML_CPP_NAMESPACE_END
