@@ -7,7 +7,7 @@
  * This file is part of libSBML.  Please visit http://sbml.org for more
  * information about SBML, and the latest version of libSBML.
  *
- * Copyright (C) 2013-2017 jointly by the following organizations:
+ * Copyright (C) 2013-2018 jointly by the following organizations:
  *     1. California Institute of Technology, Pasadena, CA, USA
  *     2. EMBL European Bioinformatics Institute (EMBL-EBI), Hinxton, UK
  *     3. University of Heidelberg, Heidelberg, Germany
@@ -149,17 +149,17 @@ public:
 
 
    /**
-   * Returns the  "ListOfFbcAssociations" in this FbcOr object.
+   * Returns the ListOfFbcAssociations in this FbcOr object.
    *
-   * @return the "ListOfFbcAssociations" attribute of this FbcOr.
+   * @return the ListOfFbcAssociations child of this FbcOr.
    */
   const ListOfFbcAssociations* getListOfAssociations() const;
 
 
   /**
-   * Returns the  "ListOfFbcAssociations" in this FbcOr object.
+   * Returns the ListOfFbcAssociations in this FbcOr object.
    *
-   * @return the "ListOfFbcAssociations" attribute of this FbcOr.
+   * @return the ListOfFbcAssociations child of this FbcOr.
    */
   ListOfFbcAssociations* getListOfAssociations();
 
@@ -170,6 +170,7 @@ public:
    * @param n the index number of the Association to get.
    *
    * @return the nth Association in the ListOfFbcAssociations within this FbcOr.
+   * If the index @p n is invalid, @c NULL is returned.
    *
    * @see getNumAssociations()
    */
@@ -182,6 +183,7 @@ public:
    * @param n the index number of the Association to get.
    *
    * @return the nth Association in the ListOfFbcAssociations within this FbcOr.
+   * If the index @p n is invalid, @c NULL is returned.
    *
    * @see getNumAssociations()
    */
@@ -196,7 +198,7 @@ public:
    * of the Association to get.
    *
    * @return the Association in the ListOfFbcAssociations
-   * with the given id or NULL if no such
+   * with the given id or @c NULL if no such
    * Association exists.
    *
    * @see getAssociation(unsigned int n)
@@ -214,7 +216,7 @@ public:
    * of the Association to get.
    *
    * @return the Association in the ListOfFbcAssociations
-   * with the given id or NULL if no such
+   * with the given id or @c NULL if no such
    * Association exists.
    *
    * @see getAssociation(unsigned int n)
@@ -538,26 +540,6 @@ public:
 
 
 
-  /** @cond doxygenLibsbmlInternal */
-
-  /**
-   * Gets the value of the "attributeName" attribute of this FbcOr.
-   *
-   * @param attributeName, the name of the attribute to retrieve.
-   *
-   * @param value, the address of the value to record.
-   *
-   * @copydetails doc_returns_success_code
-   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
-   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
-   */
-  virtual int getAttribute(const std::string& attributeName,
-                           const char* value) const;
-
-  /** @endcond */
-
-
-
   /** @cond doxygenLibsbmlInternal */
 
   /**
@@ -672,26 +654,6 @@ public:
 
 
 
-  /** @cond doxygenLibsbmlInternal */
-
-  /**
-   * Sets the value of the "attributeName" attribute of this FbcOr.
-   *
-   * @param attributeName, the name of the attribute to set.
-   *
-   * @param value, the value of the attribute to set.
-   *
-   * @copydetails doc_returns_success_code
-   * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
-   * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
-   */
-  virtual int setAttribute(const std::string& attributeName, const char*
-    value);
-
-  /** @endcond */
-
-
-
   /** @cond doxygenLibsbmlInternal */
 
   /**
@@ -716,7 +678,7 @@ public:
    *
    * @param elementName, the name of the element to create.
    *
-   * pointer to the element created.
+   * @return pointer to the element created.
    */
   virtual SBase* createChildObject(const std::string& elementName);
 
@@ -731,7 +693,7 @@ public:
    *
    * @param elementName, the name of the element to get number of.
    *
-   * unsigned int number of elements.
+   * @return unsigned int number of elements.
    */
   virtual unsigned int getNumObjects(const std::string& elementName);
 
@@ -746,9 +708,9 @@ public:
    *
    * @param elementName, the name of the element to get number of.
    *
-   * @param index, unsigned int teh index of teh object to retrieve.
+   * @param index, unsigned int the index of the object to retrieve.
    *
-   * pointer to the object.
+   * @return pointer to the object.
    */
   virtual SBase* getObject(const std::string& elementName, unsigned int index);
 
@@ -819,21 +781,20 @@ LIBSBML_CPP_NAMESPACE_END
 
 LIBSBML_CPP_NAMESPACE_BEGIN
 BEGIN_C_DECLS
-
 /**
- * Creates a new FbcOr_t structure using the given SBML @p level and
- * @p version values.
+ * Creates a new FbcOr_t using the given SBML Level, Version and
+ * &ldquo;fbc&rdquo; package version.
  *
- * @param level an unsigned int, the SBML level to assign to this
- * FbcOr_t structure.
+ * @param level an unsigned int, the SBML Level to assign to this FbcOr_t.
  *
- * @param version an unsigned int, the SBML version to assign to this
- * FbcOr_t structure.
+ * @param version an unsigned int, the SBML Version to assign to this FbcOr_t.
  *
- * @returns the newly-created FbcOr_t structure, or a null pointer if
- * an error occurred during construction.
+ * @param pkgVersion an unsigned int, the SBML Fbc Version to assign to this
+ * FbcOr_t.
  *
- * @copydetails doc_note_setting_lv
+ * @copydetails doc_note_setting_lv_pkg
+ *
+ * @copydetails doc_returned_owned_pointer
  *
  * @memberof FbcOr_t
  */
@@ -844,92 +805,232 @@ FbcOr_create(unsigned int level, unsigned int version,
 
 
 /**
- * Frees the given FbcOr_t structure.
- * 
- * @param fo the FbcOr_t structure to be freed.
+ * Frees this FbcOr_t object.
+ *
+ * @param fo the FbcOr_t structure.
  *
  * @memberof FbcOr_t
  */
 LIBSBML_EXTERN
 void
-FbcOr_free(FbcOr_t * fo);
+FbcOr_free(FbcOr_t* fo);
 
 
 /**
- * Creates a deep copy of the given FbcOr_t structure.
- * 
- * @param fo the FbcOr_t structure to be copied.
+ * Creates and returns a deep copy of this FbcOr_t object.
  *
- * @returns a (deep) copy of the given FbcOr_t structure, or a null
- * pointer if a failure occurred.
+ * @param fo the FbcOr_t structure.
+ *
+ * @return a (deep) copy of this FbcOr_t object.
+ *
+ * @copydetails doc_returned_owned_pointer
  *
  * @memberof FbcOr_t
  */
 LIBSBML_EXTERN
-FbcOr_t *
-FbcOr_clone(FbcOr_t * fo);
+FbcOr_t*
+FbcOr_clone(const FbcOr_t* fo);
 
 
+/**
+ * Adds a copy of the given FbcAssociation_t to this FbcOr_t.
+ *
+ * @param fo the FbcOr_t structure to which the FbcAssociation_t should be added.
+ *
+ * @param a the FbcAssociation_t object to add.
+ *
+ * @copydetails doc_returns_success_code
+ * @li @sbmlconstant{LIBSBML_OPERATION_SUCCESS, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_OPERATION_FAILED, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_INVALID_OBJECT, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_LEVEL_MISMATCH, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_VERSION_MISMATCH, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_PKG_VERSION_MISMATCH, OperationReturnValues_t}
+ * @li @sbmlconstant{LIBSBML_DUPLICATE_OBJECT_ID, OperationReturnValues_t}
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
 int
-FbcOr_addAssociation(FbcOr_t * fo, FbcAssociation_t * fa);
+FbcOr_addAssociation(FbcOr_t* fo, const FbcAssociation_t* a);
 
 
+/**
+ * Creates a new FbcAnd_t object, adds it to this FbcOr_t object and returns
+ * the FbcAnd_t object created.
+ *
+ * @param fo the FbcOr_t structure to which the FbcAnd_t should be added.
+ *
+ * @return a new FbcAnd_t object instance.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-FbcAnd_t *
-FbcOr_createAnd(FbcOr_t * fo);
+FbcAnd_t*
+FbcOr_createAnd(FbcOr_t* fo);
 
 
+/**
+ * Creates a new FbcOr_t object, adds it to this FbcOr_t object and returns the
+ * FbcOr_t object created.
+ *
+ * @param fo the FbcOr_t structure to which the FbcOr_t should be added.
+ *
+ * @return a new FbcOr_t object instance.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-FbcOr_t *
-FbcOr_createOr(FbcOr_t * fo);
+FbcOr_t*
+FbcOr_createOr(FbcOr_t* fo);
 
 
+/**
+ * Creates a new GeneProductRef_t object, adds it to this FbcOr_t object and
+ * returns the GeneProductRef_t object created.
+ *
+ * @param fo the FbcOr_t structure to which the GeneProductRef_t should be
+ * added.
+ *
+ * @return a new GeneProductRef_t object instance.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-GeneProductRef_t *
-FbcOr_createGeneProductRef(FbcOr_t * fo);
+GeneProductRef_t*
+FbcOr_createGeneProductRef(FbcOr_t* fo);
 
 
+/**
+ * Returns a ListOf_t * containing FbcAssociation_t objects from this FbcOr_t.
+ *
+ * @param fo the FbcOr_t structure whose ListOfAssociations is sought.
+ *
+ * @return the ListOfAssociations from this FbcOr_t as a ListOf_t *.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @see FbcOr_addAssociation()
+ * @see FbcOr_getAssociationById()
+ * @see FbcOr_getAssociation()
+ * @see FbcOr_getNumAssociations()
+ * @see FbcOr_removeAssociationById()
+ * @see FbcOr_removeAssociation()
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-ListOf_t *
-FbcOr_getListOfFbcAssociations(FbcOr_t * fo) ;
+ListOf_t*
+FbcOr_getListOfFbcAssociations(FbcOr_t* fo);
 
 
+/**
+ * Get an FbcAssociation_t from the FbcOr_t.
+ *
+ * @param fo the FbcOr_t structure to search.
+ *
+ * @param n an unsigned int representing the index of the FbcAssociation_t to
+ * retrieve.
+ *
+ * @return the nth FbcAssociation_t in the ListOfAssociations within this FbcOr.
+ * If the index @p n is invalid, @c NULL is returned.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-FbcAssociation_t *
-FbcOr_getAssociation(FbcOr_t * fo, unsigned int n);
+FbcAssociation_t*
+FbcOr_getAssociation(FbcOr_t* fo, unsigned int n);
 
 
+/**
+ * Get an FbcAssociation_t from the FbcOr_t.
+ *
+ * @param fo the FbcOr_t structure to search.
+ *
+ * @param sid the identifier of the FbcAssociation_t to
+ * retrieve.
+ *
+ * @return the FbcAssociation_t in the ListOfAssociations within this FbcOr with the given identifier.
+ *
+ * @copydetails doc_returned_unowned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
 FbcAssociation_t *
 FbcOr_getAssociationById(FbcOr_t * fo, const char * sid);
 
 
+/**
+ * Get the number of FbcAssociation_t objects in this FbcOr_t.
+ *
+ * @param fo the FbcOr_t structure to query.
+ *
+ * @return the number of FbcAssociation_t objects in this FbcOr_t.
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
 unsigned int
-FbcOr_getNumAssociations(FbcOr_t * fo);
+FbcOr_getNumAssociations(FbcOr_t* fo);
 
 
+/**
+ * Removes the nth FbcAssociation_t from this FbcOr_t and returns a pointer to it.
+ *
+ * @param fo the FbcOr_t structure to search.
+ *
+ * @param n an unsigned int representing the index of the FbcAssociation_t to
+ * remove.
+ *
+ * @return a pointer to the nth FbcAssociation_t in this FbcOr_t.
+ *
+ * @copydetails doc_returned_owned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
-FbcAssociation_t *
-FbcOr_removeAssociation(FbcOr_t * fo, unsigned int n);
+FbcAssociation_t*
+FbcOr_removeAssociation(FbcOr_t* fo, unsigned int n);
 
 
+/**
+ * Removes the FbcAssociation_t with the given identifier from this FbcOr_t and returns a pointer to it.
+ *
+ * @param fo the FbcOr_t structure to search.
+ *
+ * @param sid the identifier of the FbcAssociation_t to
+ * remove.
+ *
+ * @return a pointer to the removed FbcAssociation_t in this FbcOr_t, or NULL if no FbcAssociation_t was found.
+ *
+ * @copydetails doc_returned_owned_pointer
+ *
+ * @memberof FbcOr_t
+ */
 LIBSBML_EXTERN
 FbcAssociation_t *
 FbcOr_removeAssociationById(FbcOr_t * fo, const char * sid);
 
 
 /**
- * Predicate returning @c 1 or *c 0 depending on whether all the required
- * attributes of the given FbcOr_t structure have been set.
+ * Predicate returning @c 1 (true) if all the required attributes for this
+ * FbcOr_t object have been set.
  *
- * @param fo the FbcOr_t structure to check.
+ * @param fo the FbcOr_t structure.
  *
- * @return @c 1 if all the required attributes for this
- * structure have been defined, @c 0 otherwise.
+ * @return @c 1 (true) to indicate that all the required attributes of this
+ * FbcOr_t have been set, otherwise @c 0 (false) is returned.
  *
- * @member of FbcOr_t
+ * @memberof FbcOr_t
  */
 LIBSBML_EXTERN
 int
@@ -937,15 +1038,18 @@ FbcOr_hasRequiredAttributes(const FbcOr_t * fo);
 
 
 /**
- * Predicate returning @c 1 or *c 0 depending on whether all the required
- * sub-elements of the given FbcOr_t structure have been set.
+ * Predicate returning @c 1 (true) if all the required elements for this
+ * FbcOr_t object have been set.
  *
- * @param fo the FbcOr_t structure to check.
+ * @param fo the FbcOr_t structure.
  *
- * @return @c 1 if all the required sub-elements for this
- * structure have been defined, @c 0 otherwise.
+ * @return @c 1 (true) to indicate that all the required elements of this
+ * FbcOr_t have been set, otherwise @c 0 (false) is returned.
  *
- * @member of FbcOr_t
+ *
+ * @note The required elements for the FbcOr_t object are:
+ *
+ * @memberof FbcOr_t
  */
 LIBSBML_EXTERN
 int
